<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinterPlugin\Helpers\CloudPrinterHelper;
use CloudPrinterPlugin\Helpers\LogHelper;
use CloudPrinterPlugin\Libs\BaseController;
use CloudPrinterPlugin\Models\CPProduct;
use CloudPrinterPlugin\Services\DynamicProduct\DynamicProductService;

/**
 * Class DynamicProductOptionsController
 *
 * @copyright   2020 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class DynamicProductOptionsController extends BaseController
{
    public function __construct()
    {
        parent::__construct(CP_ABS_PATH, 'dynamic-product-options');

        // Add wrapper for dynamic options
        add_action('woocommerce_before_add_to_cart_button', [ $this, 'dynamicOptionsForGroupedProduct' ]);

        // Add selection column for grouped products list
        add_filter('woocommerce_grouped_product_columns', [ $this, 'groupedProductColumns' ]);

        // Add radio selection for grouped product
        add_filter('woocommerce_grouped_product_list_column_selection', [ $this, 'addProductSelection' ], 10, 3);

        // Add hidden quantity field for grouped product
        add_action('woocommerce_grouped_product_list_before', [ $this, 'addQuantityFieldForGroupedProduct' ]);

        add_action('woocommerce_before_calculate_totals', [$this, 'beforeCalculateTotal']);

        // Get dynamic option
        // URI: /wp-json/cloudprinter/v1/dynamic-product-options
        add_action('rest_api_init', function () {
            register_rest_route('cloudprinter/v1', '/dynamic-product-options', [
                    'methods' => 'POST',
                    'callback' => [$this, 'dynamicProductOptions'],
                    'permission_callback' => [$this, 'verifyApiRequest']
            ]);
        });

        // After add product to cart. Save the product options.
        add_filter('woocommerce_add_cart_item_data', [$this, 'saveDynamicProductOptions'], 10, 2);

        // Download product designer pdf and attach to order item
        add_action('woocommerce_checkout_create_order_line_item', [$this, 'checkoutCreateOrderLineItem'], 10, 4);

        add_filter('woocommerce_product_get_price', [$this, 'applyDynamicOptionsPrice'], 10, 2);

        // Generate temporary link to dynamic product file.
        add_action('rest_api_init', function () {
            register_rest_route('cloudprinter/v1', '/dropbox-file/link/generate', [
                'methods' => 'POST',
                'callback' => [$this, 'generateTmpLinkForDynamicProductFile'],
                'permission_callback' => [$this, 'verifyApiRequest']
            ]);
        });
    }

    public function beforeCalculateTotal($cart)
    {
        // This is necessary for WC 3.0+
        if ( is_admin() && ! defined( 'DOING_AJAX' ) ) {
            return;
        }

        // Avoiding hook repetition (when using price calculations for example)
        if ( did_action( 'woocommerce_before_calculate_totals' ) >= 2 ) {
            return;
        }

        // Loop through cart items
        foreach ( $cart->get_cart() as $item ) {
            $productId = $item['product_id'];
            $cpProduct = new CPProduct($productId);
            $productPrice = $item['data']->get_price('cart');
            if ($cpProduct->hasDynamicOptions()) {

                $cartDynamicOptions = unserialize($item['dynamicOptions']);

                if (!empty($item['dynamicOptions'])) {
                    $productDynamicOptions = $cpProduct->getDynamicOptions();

                    foreach ($productDynamicOptions as $productDynamicOption) {
                        if (count($productDynamicOption['option_values']) > 1) {
                            foreach ($productDynamicOption['option_values'] as $optionValues) {
                                if (
                                    !empty($cartDynamicOptions)
                                    && !empty($cartDynamicOptions[$productDynamicOption['option_reference']])
                                    && $cartDynamicOptions[$productDynamicOption['option_reference']] === $optionValues['value']
                                ) {
                                    $productPrice +=$optionValues['price'];
                                }
                            }
                        }
                    }
                }

            }
            $item['data']->set_price( $productPrice );
        }
    }

    /**
     * Include an option price to product price if option has only one value.
     */
    public function applyDynamicOptionsPrice($price, $product)
    {
        $productId = $product->get_id();

        $cpProduct = new CPProduct($productId);
        if ($cpProduct->hasDynamicOptions()) {
            $productDynamicOptions = $cpProduct->getDynamicOptions();

            foreach ($productDynamicOptions as $productDynamicOption) {
                if (count($productDynamicOption['option_values']) === 1) {
                    $price += $productDynamicOption['option_values'][0]['price'];
                }
            }

        }

        return $price;
    }

    /**
     * Generate temporary link to dynamic product file.
     */
    public function generateTmpLinkForDynamicProductFile($requestData)
    {
        $orderId = $requestData['order_id'];
        $dynamicProductService = new DynamicProductService();
        try {

            $dynamicProductService->generateDropboxTemporaryLinksForOrder($orderId);
            return ['success' => true];
        } catch (\Exception $e) {
            LogHelper::warning('dynamic-files', $e->getMessage());
            return ['success' => false];
        }
    }

    /**
     * Download product designer pdf and attach to order item
     */
    public function checkoutCreateOrderLineItem($item, $cart_item_key, $values, $order)
    {
        if (!empty($values['dynamicOptions'])) {
            $item->add_meta_data('_dynamicOptions', $values['dynamicOptions']);
        }
    }

    /**
     * Save dynamic options after add to cart
     */
    public function saveDynamicProductOptions($cart_item_data, $product_id)
    {
        $cpProduct = new CPProduct($product_id);
        if ($cpProduct->hasDynamicOptions()) {
            $cart_item_data['dynamicOptions'] = sanitize_text_field(serialize($_POST['cp_dynamic_options']));
        }

        return $cart_item_data;
    }

    /**
     * Add hidden quantity field for grouped products
     */
    public function addQuantityFieldForGroupedProduct()
    {
        echo "<input id='child_product_selected_quantity' type='hidden' value='1' name=''>";
    }

    /**
     * Add radio selection for grouped product
     */
    public function addProductSelection($value, $grouped_product_child)
    {
        $price = strip_tags($grouped_product_child->get_price_html());
        $disabled = $grouped_product_child->is_in_stock() ? '' : "disabled='true'";

        $elementData = [
            'price' => $price,
            'price-number' => wc_get_price_including_tax($grouped_product_child)
        ];

        $elementData = apply_filters('cloudprinter_grouped_product_data', $elementData, $grouped_product_child);

        $data = [];
        foreach ($elementData as $key => $value) {
            $data[] = "data-". $key ."='$value'";
        }

        return "<input class='child_product_selected' type='radio' value='" . $grouped_product_child->get_id() . "' name='child_product_selected' ". join(' ', $data) ." $disabled>";
    }

    /**
     * Add selection column for grouped products list
     */
    public function groupedProductColumns()
    {
        return [
            'selection',
            'label',
            'price'
        ];
    }

    /**
     * Add wrapper for grouped product dynamic options
     */
    public function dynamicOptionsForGroupedProduct()
    {
        global $product;

        $isCPProduct = CloudPrinterHelper::isCloudPrinterProduct($product->get_id());
        if ($isCPProduct && !empty($product) && $product->get_type() !== 'variation') {
            $this->render('grouped-product-dynamic-options', [
                'product' => $product
            ]);
        }
    }

    /**
     * Api endpoint. Get dynamic products for end-user side.
     */
    public function dynamicProductOptions($postData)
    {
        if (empty($postData['product_id'])) {
            return new \WP_Error( 'no_product_id', 'Undefined product id', array( 'status' => 404 ) );
        }
        $productId = wc_clean(wp_unslash($postData['product_id']));
        $product = new CPProduct($productId);

        if ($product->hasDynamicOptions()) {
            return $product->getDynamicOptions();
        }
    }

    /**
     * Verify rest api request.
     */
    public function verifyApiRequest()
    {
        $headers = array_change_key_case(getallheaders());

        return wp_verify_nonce($headers['x-wp-nonce'], 'wp_rest');
    }
}
