<?php
namespace CloudPrinterPlugin\Models;

use CloudPrinterPlugin\Helpers\CloudPrinterHelper;
use CloudPrinterPlugin\Services\DynamicOptions\DynamicOptionsService;
use CloudPrinterPlugin\Services\ProductDesigner\ProductDesignerService;

/**
 * Class CPProduct
 *
 * @copyright 2020 by CloudPrinter
 * @author    Vasyl Kushniruk <vk@cloudprinter.com>
 */
class CPProduct extends \WC_Product
{
    public function __construct($id)
    {
        // Handle the variable type product.
        $productAsPost = get_post($id, ARRAY_A, 'raw');
        if (!empty($productAsPost) && is_array($productAsPost) && !empty($productAsPost['post_type']) && $productAsPost['post_type'] == 'product_variation') {
            $id = $productAsPost['post_parent'];
        }

        parent::__construct($id);
    }

    public function isCloudprinterProduct()
    {
        return get_post_meta($this->get_id(),'_cp_is_cloud_printer_product', true) === 'yes';
    }

    /**
     * @return array|bool
     */
    public function getProductInfo()
    {
        $productInfo = $this->get_meta('cp_product_info');

        if ($productInfo) {
            return unserialize($productInfo);
        }

        return false;
    }

    /**
     * In a case of dynamic product, the product file will be uploaded on checkout step.
     * @return bool
     */
    public function isDynamicProduct()
    {
        $fileMethod = get_post_meta($this->get_id(), 'cp_file_method', true);

        return $fileMethod === 'dynamic';
    }

    /**
     * In a case of dynamic product, the product file will be uploaded on checkout step.
     * @return bool
     */
    public function isManualFileUploadSupported()
    {
        $fileMethod = get_post_meta($this->get_id(), 'cp_file_method', true);

        return $fileMethod === 'upload_later';
    }

    /**
     * In a case of static product file, the product will related to only one printed file.
     * @return bool
     */
    public function isStaticFileProduct()
    {
        $fileMethod = get_post_meta($this->get_id(), 'cp_file_method', true);

        return empty($fileMethod) || $fileMethod === 'static';
    }

    /**
     * In a case of static product file, the product will related to only one printed file.
     * @return bool
     */
    public function getFileStorage()
    {
        return  $this->get_meta('cp_file_storage');
    }

    /**
     * In a case of product designer file, the end-customer will be able to create own design.
     * @return bool
     */
    public function isProductDesignerProduct()
    {
        $fileMethod = get_post_meta($this->get_id(), 'cp_file_method', true);

        return $fileMethod === 'designer';
    }

    /**
     * Return alias of product designer.
     */
    public function getProductDesignerAlias()
    {
        return get_post_meta($this->get_id(), 'cp_product_designer', true);
    }

    /**
     * Return Instance of product designer.
     */
    public function getProductDesigner()
    {
        $productDesignerAlias = $this->getProductDesignerAlias();
        if (!$productDesignerAlias) {
            return false;
        }

        $productDesignerService = new ProductDesignerService();
        $editors = $productDesignerService->getProductDesignerEditors();

        if (!isset($editors[$productDesignerAlias])) {
            return false;
        }

        return $editors[$productDesignerAlias];
    }

    /**
     * In a case of dynamic price the price will be taken from quote.
     * @return bool
     */
    public function isDynamicPrice()
    {
        return true;
    }

    public function getProductDesignerProjectId()
    {
        return get_post_meta($this->get_id(), 'cp_product_designer_project', true);
    }

    /**
     * @return array|bool
     */
    public function getCPProductFilesInfo()
    {
        $productInfo = $this->getProductInfo();

        if ($productInfo) {
            return $productInfo['files'];
        }

        return false;
    }

    /**
     * @param $optionReference
     * @return string|null
     */
    public function getProductOptionValue($optionReference)
    {
        $options = $this->get_meta('cp_product_options');
        $options = $options ? unserialize($options) : [];

        return empty($options[$optionReference]) ? null : $options[$optionReference];
    }

    /**
     * In a case of dynamic product, the end-customer will select the product options by his own.
     * @return bool
     */
    public function hasDynamicOptions()
    {
        $productOptionType = get_post_meta($this->get_id(), 'cp_product_options_type', true);

        return $productOptionType === 'dynamic';
    }

    /**
     * Only allowed quantity values can be used
     * @return bool
     */
    public function hasPredefinedQuantities()
    {
        $predefinedQuantities = get_post_meta($this->get_id(), 'cp_predefined_quantity', true);

        if ($predefinedQuantities) {
            return true;
        }

        return false;
    }

    /**
     * Get allowed quantities values
     * @return array
     */
    public function getPossibleQuantities()
    {
        $possibleQuantities = get_post_meta($this->get_id(), 'cp_possible_quantities', true);

        if ($possibleQuantities) {
            return explode(',',$possibleQuantities);
        }

        return [];
    }

    public function getDynamicOptions()
    {
        $dynamicOptionsService = new DynamicOptionsService();
        $dynamicOptions = unserialize(get_post_meta($this->get_id(), 'cp_dynamic_product_options', true));

        return $dynamicOptionsService->formatPretty($dynamicOptions, $this);
    }

    public function getProductFiles()
    {
        $productFiles = $this->get_meta('cp_product_file');

        // Set the default schema
        if (!$productFiles) {
            $productFiles = [
                'external' => [
                    'cover' => [
                        'link' => $this->get_meta('cp_file_url_cover') ?: '',
                        'md5' => $this->get_meta('cp_file_url_cover_md5') ?: ''
                    ],
                    'book' => [
                        'link' => $this->get_meta('cp_file_url_book') ?: '',
                        'md5' => $this->get_meta('cp_file_url_book_md5') ?: ''
                    ],
                    'product' => [
                        'link' => $this->get_meta('cp_file_url_product') ?: '',
                        'md5' => $this->get_meta('cp_file_url_product_md5') ?: ''
                    ],
                    'archive' => [
                        'link' => $this->get_meta('cp_file_url_archive') ?: '',
                        'md5' => $this->get_meta('cp_file_url_zip_md5') ?: ''
                    ],
                ],
                'local' => [
                    'cover' => $this->get_meta('cp_file_cover_upload') ?: [],
                    'book' => $this->get_meta('cp_file_book_upload') ?: [],
                    'archive' => $this->get_meta('cp_file_archive_upload') ?: [],
                    'product' => $this->get_meta('cp_file_product_upload') ?: [],
                ]
            ];
        }

        return $productFiles;
    }

    public function getRequiredFilesTypes()
    {
        $requitedTypes = $this->get_meta('cp_product_file_required');
        if (!empty($requitedTypes)) {
            $requitedTypes = explode('|', $requitedTypes);
        } else {
            // support the old version products
            $productCategory = $this->get_meta('cp_product_category');
            if (CloudPrinterHelper::isBookProduct($productCategory)) {
                $requitedTypes = ['cover', 'book'];
            } elseif (CloudPrinterHelper::isPhotoPrintProduct($productCategory)) {
                $requitedTypes = ['archive'];
            } else {
                $requitedTypes = ['product'];
            }
        }

        return $requitedTypes;
    }

    public function saveProductFiles($productFiles)
    {
        $this->update_meta_data('cp_product_file', $productFiles);
        $this->save();
    }
}
